import sqlite3
from typing import List, Tuple
import os
import json
from nonebot.log import logger
from os.path import abspath, dirname
from nonebot import get_driver
from .config import Config
from .exception import *
global_config = get_driver().config
config = Config.parse_obj(global_config)
__PLUGIN_NAME = "[bilibilibot~数据库]"

class BiliDatabase():
    def __init__(self) -> None:
        # 需要从config中获得
        self.db_name = dirname(abspath(__file__)) + "/bilibili_2.db"
        self.conn = sqlite3.connect(self.db_name)
        self.init_database()
    
    def init_database(self) -> None:
        '''初始化数据库,如果不存在表则创建

        '''
        cur = self.conn.cursor()
        logger.debug(f'初始化数据库')
        
        #创建up表   
        cur.execute('''CREATE TABLE IF NOT EXISTS up
            (
                up_uid VARCHAR(20) PRIMARY KEY NOT NULL,
                up_name VARCHAR(50) NOT NULL,
                latest_update INT NOT NULL
            )
            ''')

        # 创建liver表
        cur.execute('''CREATE TABLE IF NOT EXISTS liver
            (
                liver_uid VARCHAR(20) PRIMARY KEY NOT NULL,
                liver_name VARCHAR(50) NOT NULL,
                is_live TINYINT(1) DEFAULT 0 NOT NULL,
                live_room VARCHAR(30) NOT NULL
            )
            ''')

        # 创建telegram表
        cur.execute('''CREATE TABLE IF NOT EXISTS telegram
            (
                season_id VARCHAR(20) PRIMARY KEY NOT NULL,
                telegram_title VARCHAR(50) NOT NULL,
                episode INT NOT NULL,
                is_finish TINYINT(1) DEFAULT 0 NOT NULL
            )
            ''')

        # 创建个人用户表
        cur.execute('''CREATE TABLE IF NOT EXISTS qq_user
            (
                user_id VARCHAR(20) PRIMARY KEY NOT NULL,
                user_name VARCHAR(50) NOT NULL
            )
            ''')

        # 创建群用户表
        cur.execute('''CREATE TABLE IF NOT EXISTS qq_group
            (
                group_id VARCHAR(20) PRIMARY KEY NOT NULL,
                group_name VARCHAR(50) NOT NULL
            )
            ''')

        # 创建关注up表
        cur.execute('''CREATE TABLE IF NOT EXISTS up_follower
            (
                id INTERGER PRIMARY KEY,
                up_uid VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS up_index ON up_follower (up_uid)")
        cur.execute("CREATE INDEX IF NOT EXISTS up_user_index ON up_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS up_group_index ON up_follower (group_id)")

        # 创建关注主播表
        cur.execute('''CREATE TABLE IF NOT EXISTS liver_follower
            (
                id INTERGER PRIMARY KEY,
                liver_uid VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS liver_index ON liver_follower (liver_uid)")
        cur.execute("CREATE INDEX IF NOT EXISTS liver_user_index ON liver_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS liver_group_index ON liver_follower (group_id)")

        # 创建关注节目表
        cur.execute('''CREATE TABLE IF NOT EXISTS telegram_follower
            (
                id INTERGER PRIMARY KEY,
                season_id VARCHAR(20) NOT NULL,
                user_id VARCHAR(20),
                group_id VARCHAR(20)
            )
            ''')

        # 创建索引
        cur.execute("CREATE INDEX IF NOT EXISTS tele_index ON telegram_follower (season_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS tele_user_index ON telegram_follower (user_id)")
        cur.execute("CREATE INDEX IF NOT EXISTS tele_group_index ON telegram_follower (group_id)")

        logger.debug(f'数据库初始化完毕')
        
        cur.close()
        self.conn.commit()

    def insert_info(self, sql_type: int, *args) -> bool:
        '''插入信息表的通用操作

        Args:
            type (int): 0-用户;1-群组;2-up;3-主播;4-番剧
            args: 
                0-用户:   user_id, user_name
                1-群组:   group_id, group_name
                2-up主:   up_uid, up_name, latest_update
                3-主播:   liver_uid, liver_name, is_live, live_room
                4-番剧:   season_id, telegram_title, episode, is_finish

        Returns:
            bool: 是否成功
        '''
        cur = self.conn.cursor()

        sqls = [
            "INSERT INTO qq_user(user_id, user_name) VALUES (?, ?)",
            "INSERT INTO qq_group(group_id, group_name) VALUES (?, ?)",
            "INSERT INTO up(up_uid, up_name, latest_update) VALUES (?, ?, ?)",
            "INSERT INTO liver(liver_uid, liver_name, is_live, live_room) VALUES (?, ?, ?, ?)",
            "INSERT INTO telegram(season_id, telegram_title, episode, is_finish) VALUES (?, ?, ?, ?)"
        ]
        #logger.info(f'信息表进行插入')
        
        try:
            cur.execute(sqls[sql_type], args)
        except Exception as e:
            ##logger.error(f'[]插入信息表时发生错误:\n{e}')
            self.conn.rollback()
            cur.close()
            raise BiliDatebaseError(f"数据库插入信息表时发生错误:{e.args[0]}")
        else:
            cur.close()
            self.conn.commit()
            #logger.info(f'插入信息表成功')
            return True
        
    def insert_relation(self, sql_type: int, uid: str, follower_id: str) -> bool:
        '''插入关注表

        Args:
            type (int): 0-个人关注up;1-群关注up;2-个人关注主播;3-群关注主播;4-个人关注番剧;5-群关注番剧
            uid (str): 被关注者的id
            follower_id (str): 粉丝的id

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "INSERT INTO up_follower(id, up_uid, user_id) VALUES (NULL, ?, ?)",
            "INSERT INTO up_follower(id, up_uid, group_id) VALUES (NULL, ?, ?)",
            "INSERT INTO liver_follower(id, liver_uid, user_id)  VALUES (NULL, ?, ?)",
            "INSERT INTO liver_follower(id, liver_uid, group_id) VALUES (NULL, ?, ?)",
            "INSERT INTO telegram_follower(id, season_id, user_id) VALUES (NULL, ?, ?)",
            "INSERT INTO telegram_follower(id, season_id, group_id) VALUES (NULL, ?, ?)",
        ]
        cur = self.conn.cursor()
        
        assert 0 <= sql_type < 6, "索引长度错误"
        #logger.info(f'关注表进行插入')
        try:
            cur.execute(sqls[sql_type], (uid, follower_id))
        except Exception as e:
            ##logger.error(f'插入关注表时发生错误:\n{e}')
            cur.close()
            self.conn.rollback()
            raise BiliDatebaseError(f"数据库插入关注表时发生错误:{e.args[0]}")
        else:
            #logger.info(f'插入关注表成功')
            cur.close()
            self.conn.commit()
            return True
            
    def query_info(self, sql_type: int, target_id: str) -> Tuple:
        '''查询信息表通用操作

        Args:
            sql_type (int): 0-用户;1-群组;2-up;3-主播;4-番剧
            target_id (str): up/主播/番剧/用户/群的id

        Returns:
            Tuple: 查询结果
        '''
        cur = self.conn.cursor()
        sqls = [
            "SELECT user_id, user_name FROM qq_user WHERE user_id = ?",
            "SELECT group_id, group_name FROM qq_group WHERE group_id = ?",
            "SELECT up_uid, up_name, latest_update FROM up WHERE up_uid = ?",
            "SELECT liver_uid, liver_name, is_live, live_room FROM liver WHERE liver_uid = ?",
            "SELECT season_id, telegram_title, episode FROM telegram WHERE season_id = ?"
        ]
        assert 0 <= sql_type < 5, "索引长度错误"
        #logger.info(f'查询信息表中{target_id}的信息')
        
        try:
            cur.execute(sqls[sql_type], (target_id,))
        except Exception as e:
            ##logger.error(f'查询信息表时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询信息表时发生错误:{e.args[0]}")
        else:
            result = cur.fetchone()
            return result

    def query_all(self, sql_type: int) -> List[Tuple]:
        '''获得所有的up/主播/番剧以进行更新检查

        Args:
            sql_type (int): 0-up;1-主播;2-番剧;3-用户;4-群组

        Returns:
            List[Tuple]: 查询结果
        '''

        sqls = [
            "SELECT up_uid, up_name, latest_update FROM up",
            "SELECT liver_uid, liver_name, is_live, live_room FROM liver",
            "SELECT season_id, telegram_title, episode FROM telegram WHERE is_finish IS FALSE",
            "SELECT user_id FROM qq_user",
            "SELECT group_id FROM qq_group"
        ]
        
        assert 0 <= sql_type < 5, "索引长度错误"
        cur = self.conn.cursor()

        try:
            cur.execute(sqls[sql_type])
        except Exception as e:
            ##logger.error(f'查询信息表时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询信息表时发生错误:{e.args[0]}")

        else:
            result = cur.fetchall()
            return result

    def query_user_relation(self, sql_type: int, target_id: str) -> List[Tuple]:
        '''查询个人相关的关注

        Args:
            sql_type (int): 0-关注up的所有用户;1-用户关注的所有up;2-关注主播的所有用户;3-用户关注的所有主播;4-关注番剧的所有用户;5-用户关注的所有番剧
            target_id (str): 用户/up/主播/番剧的id

        Returns:
            List[Tuple]: 查询结果列表
        '''

        sqls = [
            "SELECT user_id FROM up_follower WHERE up_uid = ?",
            "SELECT up_uid FROM up_follower WHERE user_id = ?",
            "SELECT user_id FROM liver_follower WHERE liver_uid = ?",
            "SELECT liver_uid FROM liver_follower WHERE user_id = ?",
            "SELECT user_id FROM telegram_follower WHERE season_id = ?",
            "SELECT season_id FROM telegram_follower WHERE user_id = ?"
        ]
        assert 0 <= sql_type < 6, "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'查询个人关注')
        try:
            cur.execute(sqls[sql_type], (target_id, ))
        except Exception as e:
            ##logger.error(f'查询个人关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询个人关注时发生错误:{e.args[0]}")
        else:
            result = cur.fetchall()
            return result
    
    def query_group_relation(self, sql_type: int, target_id: str) -> List[Tuple]:
        '''查询群相关的关注

        Args:
            sql_type (int): 0-关注up的所有群;1-群关注的所有up;2-关注主播的所有群;3-群关注的所有主播;4-关注番剧的所有群;5-群关注的所有番剧
            target_id (str): 群/up/主播/番剧的id

        Returns:
            List[Tuple]: 查询结果
        '''

        sqls = [
            "SELECT group_id FROM up_follower WHERE up_uid = ?",
            "SELECT up_uid FROM up_follower WHERE group_id = ?",
            "SELECT group_id FROM liver_follower WHERE liver_uid = ?",
            "SELECT liver_uid FROM liver_follower WHERE group_id = ?",
            "SELECT group_id FROM telegram_follower WHERE season_id = ?",
            "SELECT season_id FROM telegram_follower WHERE group_id = ?"
        ]
        assert 0 <= sql_type < 6, "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'查询群组关注')
        try:
            cur.execute(sqls[sql_type], (target_id, ))
        except Exception as e:
            ##logger.error(f'查询群组关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询群组关注时发生错误:{e.args[0]}")
        else:
            result = cur.fetchall()
            return result

    def query_specified_realtion(self, sql_type: int, user_id: str, target_id: str) -> bool:
        '''查询一对一的关注

        Args:
            sql_type (int): 0 个人-up;1 群组-up;2 个人-主播;3 群组-主播;4 个人-番剧;5 群组-番剧
            user_id (str): 用户/群组id
            target_id (str): up/主播/番剧id

        Returns:
            bool: 是否关注
        '''

        sqls = [
            "SELECT user_id, up_uid FROM up_follower WHERE user_id = ? AND up_uid = ?",
            "SELECT group_id, up_uid FROM up_follower WHERE group_id = ? AND up_uid = ?",
            "SELECT user_id, liver_uid FROM liver_follower WHERE user_id = ? AND liver_uid = ?",
            "SELECT group_id, liver_uid FROM liver_follower WHERE group_id = ? AND liver_uid = ?",
            "SELECT user_id, season_id FROM telegram_follower WHERE user_id = ? AND season_id = ?",
            "SELECT group_id, season_id FROM telegram_follower WHERE group_id = ? AND season_id = ?",
        ]
        
        assert 0 <= sql_type < 6, "索引长度错误"
        
        cur = self.conn.cursor()
        ##logger.info(f'查询一对一关注')
        try:
            cur.execute(sqls[sql_type], (user_id, target_id))
        except Exception as e:
            ##logger.error(f'查询一对一关注时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库查询一对一关注时发生错误:{e.args[0]}")
        else:
            result = cur.fetchone()
            if result:
                return True
            else:
                return False


    def delete_relation(self, sql_type: int, user_id: str, uid: str) -> bool:
        '''进行取关操作

        Args:
            sql_type (int): 0-用户取关up;1-群取关up;2-用户取关主播;3-群取关主播;4-用户取关番剧;5-群取关番剧
            user_id (str): 用户/群id
            uid (str): up/主播/番剧id

        Returns:
            bool: 是否成功
        '''
        sqls = [
            "DELETE FROM up_follower WHERE user_id = ? AND up_uid = ?",
            "DELETE FROM up_follower WHERE group_id = ? AND up_uid = ?",
            "DELETE FROM liver_follower WHERE user_id = ? AND liver_uid = ?",
            "DELETE FROM liver_follower WHERE group_id = ? AND liver_uid = ?",
            "DELETE FROM telegram_follower WHERE user_id = ? AND season_id = ?",
            "DELETE FROM telegram_follower WHERE group_id = ? AND season_id = ?"
        ]
        assert 0 <= sql_type < 6, "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'进行取关操作,取消{user_id}与{uid}之间的关注关系')
        
        try:
            cur.execute(sqls[sql_type], (user_id, uid))
        except Exception as e:
            ##logger.error(f'取关时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库取关时发生错误:{e.args[0]}")

        else:
            ##logger.info(f'取关成功')
            self.conn.commit()
            return True
        
    def update_info(self, sql_type: int, *args) -> bool:
        '''更新up/主播/番剧信息

        Args:
            sql_type (int): 0-更新up;1-更新主播;2-更新番剧
            args     : up-整数时间戳,uid;主播-True/False, uid;番剧-整数集数, 是否完结, season_id

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "UPDATE up SET latest_update = ? WHERE up_uid = ?",
            "UPDATE liver SET is_live = ? WHERE liver_uid = ?",
            "UPDATE telegram SET episode = ?, is_finish = ? WHERE season_id = ?"
        ]
        assert 0 <= sql_type < 3, "索引长度错误"

        cur = self.conn.cursor()
        ##logger.info(f'更新信息表')
        
        try:
            logger.debug(f"args = {args}")
            cur.execute(sqls[sql_type], args)
        except Exception as e:
            ##logger.debug(f'更新信息表时发生错误:\n{e}')
            raise e
        else:
            ##logger.info(f'更新信息表成功')
            self.conn.commit()
            return True

    def delete_info(self, sql_type: int, target_id: str) -> bool:
        '''删除用户/群组/up/主播/番剧信息

        Args:
            sql_type (int): 0-用户;1-群组;2-up;3-主播;4-番剧信息
            target_id (str): 用户/群组/up/主播/番剧id       

        Returns:
            bool: 是否成功
        '''

        sqls = [
            "DELETE FROM qq_user WHERE user_id = ?",
            "DELETE FROM qq_group WHERE group_id = ?",
            "DELETE FROM up WHERE up_uid = ?",
            "DELETE FROM liver WHERE liver_uid = ?",
            "DELETE FROM telegram WHERE season_id = ?"
        ]

        cur = self.conn.cursor()
        ##logger.info(f'从信息表中删除{target_id}的信息')
        
        try:
            cur.execute(sqls[sql_type], (target_id,))
        except Exception as e:
            ##logger.error(f'从信息表中删除时发生错误:\n{e}')
            raise BiliDatebaseError(f"数据库删除信息时发生错误:{e.args[0]}")

        else:
            ##logger.info(f'删除成功')
            self.conn.commit()
            return True
    
    def get_from_json(self) -> None:
        '''从json文件中读取数据,写入数据库
        '''
        dir_path = dirname(abspath(__file__)) + "/file/"

        if not os.path.exists(dir_path + "user"):
            return

        # 插入用户
        for json_file_name in os.listdir(dir_path + "user"):
            user_id = json_file_name.split(".")[0]
            with open(dir_path + "user/" + json_file_name, "r", encoding='utf-8') as f:
                user_info = json.load(f)
                user_name = user_info[0]

                self.insert_info(0, user_id, user_name)

                liver_list = user_info[1]
                for liver in liver_list:
                    liver_uid = liver.split()[1][:-1]
                    self.insert_relation(2, liver_uid, user_id)
                
                up_list = user_info[2]
                for up in up_list:
                    up_uid = up.split()[1][:-1]
                    self.insert_relation(0, up_uid, user_id)
                
                tele_list = user_info[3]
                for tele in tele_list:
                    tele_uid = tele.split()[1][:-1]
                    self.insert_relation(4, tele_uid, user_id)
            
            os.remove(dir_path + "user/" + json_file_name)
        os.rmdir(dir_path + "user/")
            # delete file
        
        # 插入群组
        for json_file_name in os.listdir(dir_path + "group"):
            group_id = json_file_name.split(".")[0]
            with open(dir_path + "group/" + json_file_name, "r", encoding='utf-8') as f:
                group_info = json.load(f)
                group_name = group_info[0]

                self.insert_info(1, group_id, group_name)

                liver_list = group_info[1]
                for liver in liver_list:
                    liver_uid = liver.split()[1][:-1]
                    self.insert_relation(3, liver_uid, group_id)
                
                up_list = group_info[2]
                for up in up_list:
                    up_uid = up.split()[1][:-1]
                    self.insert_relation(1, up_uid, group_id)
                
                tele_list = group_info[3]
                for tele in tele_list:
                    tele_uid = tele.split()[1][:-1]
                    self.insert_relation(5, tele_uid, group_id)  
            
            os.remove(dir_path + "group/" + json_file_name) 
        os.rmdir(dir_path + "group/")     
        # 插入up主
        for json_file_name in os.listdir(dir_path + "up"):
            up_uid = json_file_name.split(".")[0]
            with open(dir_path + "up/" + json_file_name, "r", encoding='utf-8') as f:
                up_info = json.load(f)
                up_name = up_info[0]
                up_timestamp = up_info[1]

                self.insert_info(2, up_uid, up_name, up_timestamp)
            
            os.remove(dir_path + "up/" + json_file_name)
        os.rmdir(dir_path + "up/")

        # 插入主播
        for json_file_name in os.listdir(dir_path + "stream"):
            liver_uid = json_file_name.split(".")[0]
            with open(dir_path + "stream/" + json_file_name, "r", encoding='utf-8') as f:
                liver_info = json.load(f)
                self.insert_info(3, liver_uid, liver_info[0], liver_info[1], liver_info[2])
            
            os.remove(dir_path + "stream/" + json_file_name)
        
        os.rmdir(dir_path + "stream/")
        # 插入番剧
        for json_file_name in os.listdir(dir_path + "telegram"):
            tele_uid = json_file_name.split(".")[0]
            with open(dir_path + "telegram/" + json_file_name, "r", encoding='utf-8') as f:
                tele_info = json.load(f)
                self.insert_info(4, tele_uid, tele_info[0], tele_info[1])
            
            os.remove(dir_path + "telegram/" + json_file_name)
        os.rmdir(dir_path + "telegram/")
        
bili_database = BiliDatabase()
#bili_database.get_from_json()


class BiliConnectionError(Exception):
    def __init__(self, target_type: int, target: str, reason: str):
        self.reason = reason
        self.target = target
        self.target_type = ["up主", "主播", "番剧", "短链接"][target_type]
    
    def __str__(self) -> str:
        return f"获取 {self.target_type} <{self.target}> 时发生网络错误: {self.reason}"

class BiliStatusCodeError(Exception):
    def __init__(self, target_type: int, target: str, status_code: int) -> None:
        '''网络连接状态码错误

        Args:
            target_type (int): 0-up;1-主播;2-番剧
            target (str): uid或season id
            status_code (int): 状态码
        '''
        self.status_code = status_code
        self.target = target
        self.target_type = ["up主", "主播", "番剧", "短链接"][target_type]
    
    def __str__(self) -> str:
        ret_str = f"获取 {self.target_type} <{self.target}> 时发生网络错误, 状态码为:{self.status_code}"
        return ret_str

class BiliAPIRetCodeError(Exception):
    def __init__(self, target_type: int, target: str, ret_code: int, ret_msg: str) -> None:
        '''_summary_

        Args:
            target_type (int): 0-up;1-主播;2-番剧
            target (str): uid或season id
            ret_code (int): b站接口返回的返回码
            msg (str): b站接口返回的错误信息

        Returns:
            _type_: _description_
        '''
        self.target = target
        self.target_type = ["up主", "主播", "番剧"][target_type]
        self.ret_code = ret_code
        self.ret_msg = ret_msg
        
    def __str__(self) -> str:
        ret_str = f"获取 {self.target_type} <{self.target}> 时接口返回错误\n接口错误码为:{self.ret_code}\n接口错误信息为:{self.ret_msg}"
        return ret_str

class BiliDatebaseError(Exception):
    def __init__(self, reason: str):
        self.reason = reason
    
    def __str__(self) -> str:
        return self.reason

class BiliNoLiveRoom(Exception):
    def __init__(self, liver_name: str):
        self.ret_str = f"用户 <{liver_name}> 未开通直播间" 

    def __str__(self) -> str:
        return self.ret_str

class BiliAPI404Error(Exception):
    def __init__(self) -> None:
        pass

class BiliInvalidShortUrl(Exception):
    def __init__(self, short_url: str) -> None:
        self.short_url = short_url

class BiliInvalidRoomId(Exception):
    def __init__(self, room_id: str) -> None:
        self.room_id = room_id


from nonebot import get_driver, on_command, on_message, require, permission, get_bot
from nonebot.log import logger
from nonebot.rule import regex

from nonebot.adapters.onebot.v11 import Message, MessageSegment, MessageEvent
from nonebot.adapters.onebot.v11.permission import GROUP_ADMIN, GROUP_OWNER, PRIVATE_FRIEND, GROUP_MEMBER
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.params import CommandArg
from nonebot.permission import SUPERUSER
from .config import Config

from .biliStream import *
from .biliVideo import *
from .biliTelegram import *
from .basicFunc import *
from .rule import groupMessageRule, privateMessageRule

import os
import json
import sys
import re
from typing import Union

global_config = get_driver().config
config = Config.parse_obj(global_config)

__PLUGIN_NAME = "[bilibilibot]"

ALL_PERMISSION = GROUP_ADMIN | GROUP_OWNER | PRIVATE_FRIEND | SUPERUSER

follow_liver_command = on_command("关注主播", permission=ALL_PERMISSION)
@follow_liver_command.handle()
async def follow_liver_command_handler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uid_list = args.extract_plain_text().split()
    await create_user(event)
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await follow_liver_list(event.sender.user_id, uid_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await follow_liver_list(event.group_id, uid_list, 1)
    await follow_liver_command.finish(f"关注成功:\n{success_list}\n关注失败:\n{fail_list}")

unfollow_liver_command = on_command("取关主播", aliases={"切割主播"}, permission=ALL_PERMISSION)
@unfollow_liver_command.handle()
async def unfollow_liver_command_handler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uid_list = args.extract_plain_text().split()
    await create_user(event)
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await unfollow_liver_list(event.sender.user_id, uid_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await unfollow_liver_list(event.group_id, uid_list, 1)
    await unfollow_liver_command.finish(f"取关成功:\n{success_list}\n取关失败:\n{fail_list}")

listFollowingCommand = on_command("查询关注", aliases={"查询成分"}, permission=ALL_PERMISSION)
@listFollowingCommand.handle()
async def listFollowingCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    await create_user(event)
    inputArgs = args.extract_plain_text().split()
    defaultArgs = ['直播', 'up主', '番剧']

    if isinstance(event, PrivateMessageEvent):
        user_id = event.sender.user_id
        user_type = 0
    elif isinstance(event, GroupMessageEvent):
        user_id = event.group_id
        user_type = 1
    
    exceptArgs = set(inputArgs) - set(defaultArgs)

    if len(exceptArgs) != 0:
        logger.info(f'{__PLUGIN_NAME}查询失败，存在错误参数: {exceptArgs}')
        await listFollowingCommand.finish(f"查询失败，存在错误参数:{exceptArgs}\n请正确输入命令,例如: '查询成分 直播' 或 '查询成分 直播 up主 番剧'")

    if not inputArgs:
            inputArgs = defaultArgs
    try:
        res = bili_database.query_info(user_type, user_id)
        if res:
            if user_type == 0:
                followed_up_list = bili_database.query_user_relation(1, user_id)
                followed_liver_list = bili_database.query_user_relation(3, user_id)
                followed_telegram_list = bili_database.query_user_relation(5, user_id)
            else:
                followed_up_list = bili_database.query_group_relation(1, user_id)
                followed_liver_list = bili_database.query_group_relation(3, user_id)
                followed_telegram_list = bili_database.query_group_relation(5, user_id)
            logger.debug(f'{__PLUGIN_NAME}\nup:{followed_up_list}\nliver:{followed_liver_list}\ntelegram{followed_telegram_list}')
            
            textMsg = ""
            if 'up主' in inputArgs:
                if followed_up_list:
                    textMsg += '关注的up主:\n'
                    for up_uid in followed_up_list:
                        up_uid, up_name, _ = bili_database.query_info(2, up_uid[0])
                        textMsg += '> ' + f"{up_name}(uid: {up_uid})" + '\n'
                        
                else:
                    textMsg += '无关注的up主\n'
            textMsg += '\n'

            if '直播' in inputArgs:
                if followed_liver_list:
                    textMsg += '关注的主播:\n'
                    for liver_uid in followed_liver_list:
                        liver_uid, liver_name, _, _ = bili_database.query_info(3, liver_uid[0])
                        textMsg += '> ' + f"{liver_name}(uid: {liver_uid})" + '\n'
                else:
                    textMsg += '无关注的主播\n'
            textMsg += '\n'

            if '番剧' in inputArgs:
                if followed_telegram_list:
                    textMsg += '关注的番剧\n'
                    for season_id in followed_telegram_list:
                        season_id, telegram_title, _ = bili_database.query_info(4, season_id[0])
                        textMsg += '> ' + f"{telegram_title}(season_id: ss{season_id})" + '\n'
                
                else:
                    textMsg += '无关注的番剧'
            textMsg += '\n'

            await listFollowingCommand.send(textMsg)
        else:
            await listFollowingCommand.finish("关注列表为空")
    except Exception as _:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n查询关注时发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
    await listFollowingCommand.finish()

followUpCommand = on_command("关注up", permission=ALL_PERMISSION)
@followUpCommand.handle()
async def followUpCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    await create_user(event)
    uid_list = args.extract_plain_text().split() 
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await follow_up_list(event.sender.user_id, uid_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await follow_up_list(event.group_id, uid_list, 1)
    await followUpCommand.finish(f"关注成功:\n{success_list}\n关注失败:\n{fail_list}")

unfollowUpCommand = on_command("取关up", permission=ALL_PERMISSION)
@unfollowUpCommand.handle()
async def unfollowUpCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    await create_user(event)
    uid_list = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await unfollow_up_list(event.sender.user_id, uid_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await unfollow_up_list(event.group_id, uid_list, 1)
    await unfollowUpCommand.finish(f"取关成功:\n{success_list}\n取关失败:\n{fail_list}")

followTelegramCommand = on_command("关注番剧", permission=ALL_PERMISSION)
@followTelegramCommand.handle()
async def followTelegramCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    await create_user(event)
    tele_id_list = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await follow_telegram_list(event.sender.user_id, tele_id_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await follow_telegram_list(event.group_id, tele_id_list, 1)
    await followTelegramCommand.finish(f"关注成功:\n{success_list}\n关注失败:\n{fail_list}")

unfollowTelegramCommand = on_command("取关番剧", permission=ALL_PERMISSION)
@unfollowTelegramCommand.handle()
async def unfollowTelegramCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    await create_user(event)
    season_id_list = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        success_list, fail_list = await unfollow_telegram_list(event.sender.user_id, season_id_list, 0)
    if isinstance(event, GroupMessageEvent):
        success_list, fail_list = await unfollow_telegram_list(event.group_id, season_id_list, 1)
    await unfollowTelegramCommand.finish(f"取关成功:\n{success_list}\n取关失败:\n{fail_list}")

followUpByShare = on_message(
    rule=regex('\[CQ:json,[\w\W]*"appid":100951776[\w\W]*space.bilibili.com[\w\W]*[\w\W]*\]') & privateMessageRule, 
    permission=PRIVATE_FRIEND
    )
@followUpByShare.handle()
async def upShareHandler(event: PrivateMessageEvent):
    '''响应用户分享up主空间连接

    Args:
        event (PrivateMessageEvent): 消息事件
    '''
    await create_user(event)
    sJson = event.get_message()[-1].get('data')
    data = json.loads(sJson['data'])
    uid = data['meta']['news']['jumpUrl'].split('?')[0].split('/')[-1]
    
    success_list, fail_list = await follow_up_list(event.sender.user_id, [uid], 0)
    
    if success_list:
        await followUpByShare.finish(f"关注up成功: <{success_list[0]}>")
    elif fail_list:
        await followUpByShare.finish(f"关注up失败: <{fail_list[0]}>")

followStreamerByShare = on_message(
    rule=regex('^\[CQ:json,[\w\W]*"appid":100951776[\w\W]*live.bilibili.com[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
    )
@followStreamerByShare.handle()
async def streamerShareHandler(event: PrivateMessageEvent):
    '''响应个人用户的直播间分享

    Args:
        event (PrivateMessageEvent): 消息事件
    '''
    try:
        await create_user(event)
        sJson = event.get_message()[-1].get('data')
        data = json.loads(sJson['data'])
        roomNumber = data['meta']['news']['jumpUrl'].split('?')[0].split('/')[-1]

        uid, _ = await bili_client.init_liver_info_by_room_id(roomNumber)

        success_list, fail_list = await follow_liver_list(event, event.sender.user_id, [uid], 0)

        if success_list:
            await followUpByShare.finish(f"关注主播成功: <{success_list[0]}>")
        elif fail_list:
            await followUpByShare.finish(f"关注主播失败: <{fail_list[0]}> ")
        
    except nonebot.exception.FinishedException:
        pass
    except BiliInvalidRoomId:
        await followStreamerByShare.finish('关注失败: 无效的房间号')
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        logger.error(f'{__PLUGIN_NAME}获取主播 <{uid}> 信息时发生错误')
        logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
        await followStreamerByShare.finish('关注失败: 连接错误')

followTelegramByShare = on_message(
    rule = regex('^\[CQ:json[\w\W]*"appid":100951776[\w\W]*www.bilibili.com\/bangumi\/play\/[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
)
@followTelegramByShare.handle()
async def telegramShareHandler(event: PrivateMessageEvent):
    '''响应用户分享番剧页面

    Args:
        event (PrivateMessageEvent): 消息事件
    '''
    await create_user(event)
    sJson = event.get_message()[-1].get('data')
    data = json.loads(sJson['data'])
    epID = data['meta']['detail_1']['qqdocurl'].split('?')[0].split('/')[-1]
    
    success_list, fail_list = await follow_telegram_list(event.sender.user_id, [epID], 0)
    
    if success_list:
        await followUpByShare.finish(f"关注番剧成功: <{success_list[0]}>")
    elif fail_list:
        await followUpByShare.finish(f"关注番剧失败: <{fail_list[0]}> ")

follow_by_share_short_url = on_message(
    rule = regex('^\[CQ:json[\w\W]*"appid":100951776[\w\W]*b23.tv[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
)
@follow_by_share_short_url.handle()
async def short_url_handler(event: PrivateMessageEvent):
    try:
        await create_user(event)
        sJson = event.get_message()[-1].get('data')
        shortLink = re.search("https:\/\/b23.tv\/\w+", sJson['data'])
        short_url = shortLink.group()
        logger.debug(f"get short url = {short_url}")
        
        msg_type = 0
        user_id = event.sender.user_id
        id_type, target_uid = await bili_client.parse_short_url(short_url)
        #isSuccess, id_type, uid = await parseB23Url(short_url)
        if id_type == 0:
            success_list, fail_list = await follow_up_list(user_id, [target_uid], msg_type)
            if success_list:
                await follow_by_share_short_url.finish(f"关注成功: {success_list[0]}")
            elif fail_list:
                await follow_by_share_short_url.finish(f"关注失败: {fail_list[0]}")
            
        elif id_type == 1:
            liver_uid, _ = await bili_client.init_liver_info_by_room_id(target_uid)
            success_list, fail_list = await follow_liver_list(user_id, [liver_uid], msg_type)
            if success_list:
                await follow_by_share_short_url.finish(f"关注成功: {success_list[0]}")
            elif fail_list:
                await follow_by_share_short_url.finish(f"关注失败: {fail_list[0]}")

        elif id_type == 2:
            success_list, fail_list = await follow_telegram_list(user_id, [target_uid], msg_type)
            if success_list:
                await follow_by_share_short_url.finish(f"关注成功: {success_list[0]}")
                
            elif fail_list:           
                await follow_by_share_short_url.finish(f"关注失败: {fail_list[0]}")
        else:
            await follow_by_share_short_url.finish(f"关注失败: 非法短链接{short_url}")
    except nonebot.exception.FinishedException:
        pass
    except BiliInvalidShortUrl:
        await follow_by_share_short_url.finish('关注失败: 非法或不支持的短链接')
    except Exception as _:
        ex_type, ex_val, _ = sys.exc_info()
        logger.error(f'{__PLUGIN_NAME}【错误报告】\n解析短链接 <{short_url}> 时发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n{traceback.format_exc()}')
        await follow_by_share_short_url.finish('关注失败: 连接错误')

helpCommand = on_command("help", permission=ALL_PERMISSION, aliases={'帮助'})
@helpCommand.handle()
async def sendHelpMsg(event: MessageEvent):
    await create_user(event)
    helpMsg = ""
    with open(f'{PackagePath}/file/source/help.json', 'r', encoding='utf-8') as f:
        helpMsg = json.load(f)
    await helpCommand.finish(helpMsg)

publicBroacast = on_command("broacast", aliases={'广播'}, permission=permission.SUPERUSER)
@publicBroacast.handle()
async def sendBroacast(event: MessageEvent):
    announcement = ""
    announcementPath = f'{PackagePath}/file/source/announcement.json'
    if os.path.exists(announcementPath):
        with open(announcementPath, 'r', encoding='utf-8') as f:
            announcement = json.load(f)
            
        users = GetAllUser()
        await SendMsgToUsers(announcement, users)

        groups = GetAllGroup()
        await SendMsgToGroups(announcement, groups)

        await publicBroacast.finish("公告发送成功")
    else:
        logger.debug(f'{__PLUGIN_NAME}公告文件不存在')
        await publicBroacast.finish("公告发送失败: 公告文件不存在") 

require("nonebot_plugin_apscheduler")
from nonebot_plugin_apscheduler import scheduler
logger.debug(f'{__PLUGIN_NAME}注册定时任务')
scheduler.add_job(check_bili_live, "interval", minutes=1, id="biliStream", misfire_grace_time=90)
scheduler.add_job(check_up_update, "interval", minutes=5, id="biliUp", misfire_grace_time=90)
scheduler.add_job(check_telegram_update, "interval", minutes=10, id="biliTele", misfire_grace_time=90)


from .db import bili_database
from .exception import BiliConnectionError, BiliDatebaseError
import httpx
import asyncio
import json
from typing import Tuple, List
import sys
import traceback
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from .basicFunc import *
from .bili_client import bili_client

__PLUGIN_NAME = "[bilibilibot~视频]"
BASEURL = 'https://api.bilibili.com/x/space/arc/search?mid={}&ps=30&tid=0&pn=1&keyword=&order=pubdate&jsonp=jsonp'
USERINFOURL = 'https://api.bilibili.com/x/space/acc/info?mid={}&jsonp=jsonp'


# 视频
async def check_up_update() -> None:
    """
    @description  :
    检查关注UP主是否更新新视频，如果更新则通知用户并写入文件
    ---------
    @param  :
    -------
    @Returns  :
    -------
    """
    up_list = bili_database.query_all(0)
    schedBot = nonebot.get_bot()
    #assert status == True, "数据库发生错误"
    param_list = [[up_info[0], up_info[2]] for up_info in up_list]
    results = await asyncio.gather(
        *[bili_client.get_latest_video(uid, latest_timestamp) for uid, latest_timestamp in param_list],
        return_exceptions=True
    )
    
    for i in range(len(up_list)):
        if isinstance(results[i], tuple):
            if results[i][0] is True:
                logger.info(f'{__PLUGIN_NAME}检测到up主<{up_list[i][1]}>更新了视频')
                textMsg = f"【B站动态】\n <{up_list[i][1]}> 更新了视频\n标题: {results[i][2]}\n链接: https://www.bilibili.com/video/{results[i][1]}"
                bili_database.update_info(0, results[i][3], up_list[i][0])

                user_list = bili_database.query_user_relation(0, up_list[i][0])
                for user_id in user_list:
                    await schedBot.send_msg(message=textMsg + MessageSegment.image(results[i][4]), user_id=user_id[0])
                
                group_list = bili_database.query_group_relation(0, up_list[i][0])
                for group_id in group_list:
                    await schedBot.send_msg(message=textMsg + MessageSegment.image(results[i][4]), group_id=group_id[0])
            
        elif isinstance(results[i], (BiliAPIRetCodeError, BiliStatusCodeError, BiliConnectionError)):
            exception_msg = f'[错误报告]\n检测up主 <{up_list[i][1]}> 更新情况时发生错误\n错误类型: {type(results[i])}\n错误信息: {results[i]}'
            logger.error(f"[{__PLUGIN_NAME}]" + exception_msg)

async def follow_up(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户或群关注up，修改数据库

    Args:
        uid (str): up的uid
        user_id (str): 用户的qq或群号
        user_type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    # 处理参数错误
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数<{uid}>')
        return (False, uid + "(错误参数)")
    
    uid = str(int(uid))
    
    try:
        result = bili_database.query_info(2, uid)
        
        # up信息不存在于数据库,对数据库进行更新
        if not result:
            up_name, latest_timestamp = await bili_client.init_up_info(uid)

            if up_name:
                bili_database.insert_info(2, uid, up_name, latest_timestamp)
                bili_database.insert_relation(0 + user_type, uid, user_id)
                logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注主播 <{up_name}> 成功")
                return (True, up_name + f"(uid: {uid})")
            else:
                logger.info(f'{__PLUGIN_NAME}up({uid})不存在，请检查uid')
                return (False, uid + "(uid错误)")
        # 处理已关注
        result1 = bili_database.query_specified_realtion(0 + user_type, user_id, uid)
        
        if result1:
            logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 已经关注了up <{result[1]}>")
            return (False, result[1] + "(已关注)")
    
        # 进行关注
        bili_database.insert_relation(0 + user_type, uid, user_id)
        logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注up <{result[1]}> 成功")
        return (True, result[1] + f"(uid: {uid})")
    except BiliAPI404Error:
        return (False, uid + "(uid错误)")
    except (BiliConnectionError, BiliAPIRetCodeError, BiliStatusCodeError):
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n获取up主 <{uid}> B站信息发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(网络错误)")
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注up主 <{uid}>时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注up主 <{uid}>时发生意料之外的错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(未知错误,请查看日志)")

async def unfollow_up(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据个人用户或群取关up主，修改up文件

    Args:
        uid (str): up的uid
        user_id (str): 用户的qq号/群号
        user_type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    # 处理参数错误
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数 <{uid}>')
        return (False, uid + "(错误参数)")
    try:
    # 处理未关注
        result = bili_database.query_specified_realtion(0 + user_type, user_id, uid)
        if not result:
            logger.info(f'{__PLUGIN_NAME}用户/群 <{user_id}> 未关注up <{uid}>')
            return (False, uid + "(未关注)")
        
        # 进行取关
        bili_database.delete_relation(0 + user_type, user_id, uid)
        logger.info(f'{__PLUGIN_NAME}用户/群 <{user_id}> 取关up <{uid}> 成功')

        return (True, bili_database.query_info(2, uid)[1]  + f"(uid: {uid})") 
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n取关up主 <{uid}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")

async def follow_up_list(
    user_id: int, 
    uid_list: List[str], 
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群关注up主

    Args:
        user_id (int): qq号/群号
        uid_list (List[str]): up主的uid
        user_type (int): 0-个人用户，1-群        

    Returns:
        List[List[str]]: [[关注成功列表]，[关注失败列表]]
    '''
    successList = []
    failList = []
    
    for uid in uid_list:
        isSuccess, s = await follow_up(uid, str(user_id), user_type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)

    return [successList, failList]

async def unfollow_up_list(
    user_id: int, 
    uid_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群取关up主

    Args:
        user_id (int): qq号/群号
        uid_list (List[str]): 取关的up主
        user_type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    successList = []
    failList = []

    for uid in uid_list:
        isSuccess, s = await unfollow_up(uid, user_id, user_type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)
    
    return [successList, failList]


import json
from typing import Tuple, List
import sys
import traceback
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .basicFunc import *
from .db import bili_database
from .exception import *
from random import choice
import asyncio
from .bili_client import bili_client

__PLUGIN_NAME = "[bilibilibot~直播]"
BASEURL = 'https://api.bilibili.com/x/space/arc/search?mid={}&ps=30&tid=0&pn=1&keyword=&order=pubdate&jsonp=jsonp'
USERINFOURL = 'https://api.bilibili.com/x/space/acc/info?mid={}&jsonp=jsonp'
LIVEINFOURL = 'https://api.live.bilibili.com/xlive/web-room/v2/index/getRoomPlayInfo?room_id={}&protocol=0,1&format=0,1,2&codec=0,1&qn=0&platform=web&ptype=8'

async def check_bili_live() -> None:
    """
    @description  :
    检查数据库中所有主播的开播状态
    如果关注的主播开播，则通知所有关注的用户
    如果主播开播状态改变,则更新数据库
    ---------
    @param  :
    -------
    @Returns  :
    -------
    """
    liver_list = bili_database.query_all(1)
    logger.debug(f'{__PLUGIN_NAME}主播数据库:{liver_list}')
    sched_bot = nonebot.get_bot()
    
    results = await asyncio.gather(
        *[bili_client.get_live_status(liver_info[0], liver_info[3]) for liver_info in liver_list],
        return_exceptions=True
    )
    logger.debug(f'{__PLUGIN_NAME}直播状态结果:{results}')
    
    for i in range(len(liver_list)):
        if isinstance(results[i], tuple):
            if results[i][0] and not liver_list[i][2]:
                logger.info(f'[{__PLUGIN_NAME}]检测到主播 <{liver_list[i][1]}> 已开播！')
                text_msg = '【直播动态】\n<{}>正在直播!\n标题: {}\n链接: {}'.format(liver_list[i][1], results[i][1], f"https://live.bilibili.com/{liver_list[i][3]}")
                
                logger.info(f'[{__PLUGIN_NAME}]向粉丝发送开播通知')
                bili_database.update_info(1, 1, liver_list[i][0])

                user_list = bili_database.query_user_relation(2, liver_list[i][0])
                for user in user_list:
                    await sched_bot.send_msg(message=text_msg + MessageSegment.image(results[i][2]), user_id=user[0])
                
                group_list = bili_database.query_group_relation(2, liver_list[i][0])
                for group in group_list:
                    await sched_bot.send_msg(message=text_msg + MessageSegment.image(results[i][2]), group_id=group[0])
            elif not results[i][0] and liver_list[i][2]:
                logger.info(f"[{__PLUGIN_NAME}]检测到主播 <{liver_list[i][1]}> 已下播")
                bili_database.update_info(1, 0, liver_list[i][0])
        elif isinstance(results[i], (BiliAPIRetCodeError, BiliStatusCodeError, BiliConnectionError)):
            exception_msg = f'[错误报告]\n检测主播 <{liver_list[i][1]}> 开播情况时发生错误\n错误类型: {type(results[i])}\n错误信息: {results[i]}'
            logger.error(f"[{__PLUGIN_NAME}]" + exception_msg)
    
async def follow_liver(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户/群关注主播，修改数据库

    Args:
        uid (str): 主播的uid
        user_id (str): 用户的uid或群号
        user_type (int): 0-用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，主播名(uid) | 主播uid(失败原因)]
    '''
    
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数 <{uid}>')
        return (False, uid + "(错误参数)")
    uid = str(int(uid))
    try:
        result = bili_database.query_info(3, uid)
        
        if not result:
            # 主播在数据库中无信息,进行数据库更新
            liver_name, room_url = await bili_client.init_liver_info(uid)
        
            if liver_name and room_url:
                bili_database.insert_info(3, uid, liver_name, False, room_url)
                bili_database.insert_relation(2 + user_type, uid, user_id)
                logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注主播 <{liver_name}> 成功")
                return (True, liver_name + f"(uid: {uid})")
            elif liver_name:
                logger.debug(f'{__PLUGIN_NAME}主播 <{liver_name}>未开通直播间')
                return (False, uid + "(该用户未开通直播间)")
            else:
                logger.debug(f'{__PLUGIN_NAME}主播 <{uid}> 不存在')
                return (False, uid + "(非法uid)")
        
        if bili_database.query_specified_realtion(2 + user_type, user_id, uid):
            logger.debug(f'{__PLUGIN_NAME}主播 <{result[1]}> 已关注')
            return (False, uid + "(已关注)")
   
        bili_database.insert_relation(2 + user_type, uid, user_id)
        logger.info(f"{__PLUGIN_NAME}用户/群 <{user_id}> 关注主播 <{result[1]}> 成功")
        return (True, result[1] + f"(uid: {uid})")
    except (BiliConnectionError, BiliAPIRetCodeError, BiliStatusCodeError):
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n获取主播 <{uid}> B站信息发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(网络错误)")
    except BiliNoLiveRoom:
        return (False, uid + "(未开通直播间)")
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注主播 <{uid}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注主播 <{uid}> 时发生意料之外的错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(未知错误,请查看日志)")

async def unfollower_liver(uid: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户取关主播，修改主播文件

    Args:
        uid (str): 主播的uid
        user_id (int):QQ号/群号
        user_type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，主播名 | 失败原因]
    '''
    if not uid.isdigit():
        logger.error(f'{__PLUGIN_NAME}存在错误参数 <{uid}>')
        return (False, uid + "(错误参数)")
    try:
        result = bili_database.query_specified_realtion(2 + user_type, user_id, uid)
        if result:
            liver_info = bili_database.query_info(3, uid)
            bili_database.delete_relation(2 + user_type, user_id, uid)
            # 如果没人关注,删除主播
            return (True, liver_info[1] + f"(uid: {uid})")
        else:
            logger.debug(f'{__PLUGIN_NAME}用户 <{user_id}> 未关注主播<{uid}>')
            return (False, uid + "(未关注)")
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n取关主播 <{uid}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"{__PLUGIN_NAME}" + exception_msg + traceback.format_exc())
        return (False, uid + "(数据库错误)")

async def follow_liver_list(
    user_id: int, 
    uid_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''用户/群对主播进行关注

    Args:
        user_id (int): 用户qq或群号
        uid_list (List[str]): 关注的主播uid
        user_type (int): 0-用户, 1-群

    Returns:
        List[List[str]]: [[关注成功], [关注失败]]
    '''

    success_list = []
    fail_list = []

    for uid in uid_list:
        isSuccess, s = await follow_liver(uid, str(user_id), user_type)
        if isSuccess:
            success_list.append(s)
        else:
            fail_list.append(s)

    return [success_list, fail_list]


async def unfollow_liver_list(
    user_id: int, 
    uid_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''用户/群对主播取关

    Args:
        user_id (int): 用户qq/群号
        uid_list (List[str]): 取关主播列表
        user_type (int): 0-用户, 1-群

    Returns:
        List[List[str]]: [成功列表，失败列表]
    '''

    success_list = []
    fail_list = []

    for uid in uid_list:
        isSuccess, s = await unfollower_liver(uid, str(user_id), user_type)
        if isSuccess:
            success_list.append(s)
        else:
            fail_list.append(s)
    
    return [success_list, fail_list]


import json
from typing import Tuple, List, Union
import sys
import os
import asyncio
import traceback
from unittest import result
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .basicFunc import *
from .exception import *
import httpx
from .db import bili_database
from .bili_client import bili_client
__PLUGIN_NAME = "[bilibilibot~番剧]"
biliTeleInfoUrl = 'https://api.bilibili.com/pgc/web/season/section?season_id={}'
GETSEASONIDAPI = 'https://api.bilibili.com/pgc/view/web/season?ep_id={}'
GETEPISODESAPI = 'https://api.bilibili.com/pgc/web/season/section?season_id={}'
    
async def check_telegram_update():
    """
    @description  :
    检查数据库中的每一个影视节目是否更新，如果更新则向用户发送通知，并且更新文件
    ---------
    @param  :
    无
    -------
    @Returns  :
    无
    -------
    """
    
    telegram_list = bili_database.query_all(2)
    sched_bot = nonebot.get_bot()
    season_id_list = [[telegram[0], telegram[2]] for telegram in telegram_list]
    results = await asyncio.gather(
        *[bili_client.get_telegram_latest_episode(season_id, episode) for season_id, episode in season_id_list],
        return_exceptions=True
    )

    for i in range(len(telegram_list)):
        if isinstance(results[i], tuple):
            if results[i][0] is True:
                logger.info(f'[{__PLUGIN_NAME}]检测到影视剧 <{telegram_list[i][1]}> 更新')
                bili_database.update_info(2, results[i][1], results[i][5], telegram_list[i][0])
                
                text_msg = "【B站动态】\n《{}》已更新第{}集\n标题: {}\n链接: {}\n".format(
                        telegram_list[i][1], results[i][1], results[i][2], results[i][3]
                    )
                cover_msg = MessageSegment.image(results[i][4])
                reported_msg = text_msg + cover_msg
                logger.info(f'[{__PLUGIN_NAME}]向关注用户发送更新通知')

                # 通知用户
                user_list = bili_database.query_user_relation(4, telegram_list[i][0])
                for user in user_list:
                    await sched_bot.send_msg(message=reported_msg, user_id=user[0])

                group_list = bili_database.query_group_relation(4, telegram_list[i][0])
                for group in group_list:
                    await sched_bot.send_msg(message=reported_msg, group_id=group[0])
        
        elif isinstance(results[i], (BiliAPIRetCodeError, BiliStatusCodeError, BiliConnectionError)):
            exception_msg = f'[错误报告]\n检测番剧 <{telegram_list[i][1]}> 更新情况时发生错误\n错误类型: {type(results[i])}\n错误信息: {results[i]}'
            logger.error(f"[{__PLUGIN_NAME}]" + exception_msg)
        
async def follow_telegram(id_prefix: str, telegram_id: int, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户关注节目，修改节目的文件

    Args:
        id_prefix (str): ep | ss | md
        telegram_id (int): 节目的id
        user_id (str): 用户的qq号/群号
        user_type (int): 0-个人用户，1-群号

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    try:
        if id_prefix == "ep":
            res = await bili_client.init_telegram_info_by_ep_id(telegram_id)
        elif id_prefix == "ss":
            res = await bili_client.init_telegram_info_by_season_id(telegram_id)
        elif id_prefix == "md":
            res = await bili_client.init_telegram_info_by_media_id(telegram_id)
        else:
            return (False, telegram_id + "(番剧id错误)")
        
        season_id, telegram_title, episode, is_finish = res
        telegram_info = bili_database.query_info(4, season_id)

        # 如果数据库无番剧信息,则插入番剧信息
        if not telegram_info:
            bili_database.insert_info(4, season_id, telegram_title, episode, is_finish)

        result1 = bili_database.query_specified_realtion(4 + user_type, user_id, season_id)
        if result1:
            logger.debug(f'[{__PLUGIN_NAME}]用户 <{user_id}> 已关注节目 <{telegram_title}>')
            return (False, telegram_title + "(已关注)")
        
        bili_database.insert_relation(4 + user_type, season_id, user_id)
        logger.info(f"[{__PLUGIN_NAME}]用户/群 <{user_id}> 关注番剧 <{telegram_title}> 成功")
        return (True, telegram_title + f"(season_id: {season_id})")
    
    except BiliAPI404Error:
        return (False, f"{id_prefix}{telegram_id}" + "(番剧id错误)")
    except (BiliConnectionError, BiliAPIRetCodeError, BiliStatusCodeError):
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n获取番剧 <{id_prefix}{telegram_id}> 信息发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"[{__PLUGIN_NAME}]" + exception_msg + traceback.format_exc())
        return (False, f"{id_prefix}{telegram_id}" + "(网络错误)")
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注番剧 <{season_id}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"[{__PLUGIN_NAME}]" + exception_msg + traceback.format_exc())
        return (False, season_id + "(数据库错误)")
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n关注番剧 <{id_prefix}{telegram_id}> 时发生意料之外的错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"[{__PLUGIN_NAME}]" + exception_msg + traceback.format_exc())
        return (False, f"{id_prefix}{telegram_id}" + "(未知错误,请查看日志)")
        
async def unfollow_telegram(season_id: str, user_id: str, user_type: int) -> Tuple[bool, str]:
    '''根据用户/群取关节目，修改节目文件

    Args:
        season_id (str): 节目的ID
        user_id (str): 用户qq号/群号
        user_type (int): 0-个人用户，1-群号

    Returns:
        Tuple[bool, str]: [是否成功, 信息]
    '''

    if not season_id.isdigit():
        return (False, season_id + "(错误参数)")
    
    try:
        result = bili_database.query_specified_realtion(4 + user_type, user_id, season_id)
        
        # 处理未关注
        if not result:
            logger.info(f'[{__PLUGIN_NAME}]用户/群 <{user_id}> 未关注番剧 <{season_id}>')
            return (False, season_id + "(未关注)")

        # 进行取关
        bili_database.delete_relation(4 + user_type, user_id, season_id)
        logger.info(f'[{__PLUGIN_NAME}]用户/群 <{user_id}> 取关番剧 <{season_id}> 成功')
        return (True, bili_database.query_info(4, season_id)[1]  + f"(season_id: {season_id})") 
    except BiliDatebaseError:
        ex_type, ex_val, _ = sys.exc_info()
        exception_msg = f'【错误报告】\n取关番剧 <{season_id}> 时数据库发生错误\n错误类型: {ex_type}\n错误值: {ex_val}\n'
        logger.error(f"[{__PLUGIN_NAME}]" + exception_msg + traceback.format_exc())
        return (False, season_id + "(数据库错误)")
    
async def follow_telegram_list(
    user_id: int, 
    telegram_id_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群关注番剧

    Args:
        user_id (int): qq号/群号
        telegram_id_list (List[str]): 关注的番剧号
        user_type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    # 在该函数中同时对telegram_id, season_id, media_id进行区分和统一处理
    success_list = []
    fail_list = []
    prefix_type = ('ep', 'ss', "md")
    valid_telegram_id_list = []
    for telegram_id in telegram_id_list:
        logger.debug(f'{__PLUGIN_NAME}telegram_id = {telegram_id}')
        
        if telegram_id[:2] not in prefix_type or not telegram_id[2:].isdigit():
            fail_list.append(telegram_id + "(错误参数)")
        else:
            valid_telegram_id_list.append(telegram_id)
    logger.debug(f'{__PLUGIN_NAME}valid list = {valid_telegram_id_list}')
    
    results = await asyncio.gather(
        *[follow_telegram(telegram_id[:2], int(telegram_id[2:]), user_id, user_type) for telegram_id in valid_telegram_id_list],
        return_exceptions=True
    )

    for is_success, msg in results:
        if is_success:
            success_list.append(msg)
        else:
            fail_list.append(msg)
    
    return [success_list, fail_list] 

async def unfollow_telegram_list(
    user_id: int, 
    season_id_list: List[str],
    user_type: int
    ) -> List[List[str]]:
    '''个人用户/群取关番剧

    Args:
        user_id (int): qq号/群号
        season_id_list (List[str]): 取关的番剧号
        user_type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    success_list = []
    fail_list = []

    for season_id in season_id_list:
        isSuccess, s = await unfollow_telegram(season_id, user_id, user_type)
        if isSuccess:
            success_list.append(s)
        else:
            fail_list.append(s)
    
    return [success_list, fail_list]                 
                

            
            

        
    
            



from typing import List, Tuple, Union
import re
import httpx
import os
import sys
import traceback
from nonebot import get_bot
from nonebot.log import logger
from os.path import abspath, dirname
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .db import bili_database
from .exception import *
PackagePath =  dirname(abspath(__file__))
__PLUGIN_NAME = "[bilibilibot~基础]"

def GetAllUser() -> List[str]:
    """
    @description  :
    获取所有用户
    ---------
    @param  :
    无
    -------
    @Returns  :
    返回所有用户qq号组成的列表
    -------
    """
    qq_users = bili_database.query_all(3)
    return [i[0] for i in qq_users]

def GetAllGroup() -> List[str]:
    """
    @description  :
    获取所有群
    ---------
    @param  :
    无
    -------
    @Returns  :
    返回所有群号组成的列表
    -------
    """

    qq_groups = bili_database.query_all(4)
    return [i[0] for i in qq_groups]

async def SendMsgToUsers(msg: str, users: List[str]):
    """
    @description  :
    向所有用户发送公告
    ---------
    @param  :
    msg: 内容
    users: 用户的qq列表
    -------
    @Returns  :
    无
    -------
    """
    
    bot = get_bot()
    for user in users:
        await bot.send_msg(message=msg, user_id = user)
    
async def SendMsgToGroups(msg: str, groups: List[str]):
    '''向所有群组发送公告

    Args:
        msg (str): 公告内容
        groups (List[str]): 群组列表
    '''
    
    bot = get_bot()
    for group in groups:
        await bot.send_msg(message=msg, group_id = group)

async def create_user(event: Union[PrivateMessageEvent, GroupMessageEvent]) -> None:
    '''接受消息后,创建用户

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
    '''

    user_type = 0 if isinstance(event, PrivateMessageEvent) else 1
    user_id = event.sender.user_id if user_type == 0 else event.group_id
    try:
        user_info = bili_database.query_info(user_type, str(user_id))
        if not user_info:
            logger.info(f'{__PLUGIN_NAME}用户{user_id}不存在于数据库,即将创建')
            name = event.sender.nickname
            if user_type == 1:
                bot = get_bot()
                group_info = await bot.get_group_info(group_id=user_id)
                name = group_info["group_name"]
            bili_database.insert_info(user_type, user_id, name)
    except Exception as _:
            ex_type, ex_val, _ = sys.exc_info()
            exception_msg = '【错误报告】\n创建用户时发生错误\n错误类型: {}\n错误值: {}\n'.format(ex_type, ex_val)
            logger.error(f"{__PLUGIN_NAME}\n" + exception_msg + traceback.format_exc())


def CheckDir():
    """
    @description  :
    检查插件运行所需要的文件目录是否存在，不存在则创建
    ---------
    @param  :
    
    -------
    @Returns  :
    
    -------
    """
    
    baseDir = f'{PackagePath}/file/'

    if not os.path.exists(baseDir + 'source'):
        logger.debug(f'{__PLUGIN_NAME}source文件夹不存在')
        os.makedirs(baseDir + 'source')
        
    if not os.path.exists(baseDir + 'stream'):
        logger.debug(f'{__PLUGIN_NAME}stream文件夹不存在')
        os.makedirs(baseDir + 'stream')
    
    if not os.path.exists(baseDir + 'telegram'):
        logger.debug(f'{__PLUGIN_NAME}telegram文件夹不存在')
        os.makedirs(baseDir + 'telegram')
    
    if not os.path.exists(baseDir + 'up'):
        logger.debug(f'{__PLUGIN_NAME}up文件夹不存在')
        os.makedirs(baseDir + 'up')
        
    if not os.path.exists(baseDir + 'user'):
        logger.debug(f'{__PLUGIN_NAME}user文件夹不存在')
        os.makedirs(baseDir + 'user')
    
    if not os.path.exists(baseDir + 'group'):
        logger.debug(f'{__PLUGIN_NAME}group文件夹不存在')
        os.makedirs(baseDir + 'group')




import httpx
from random import choice
from lxml import etree
import asyncio
import json
from typing import List, Tuple
from .exception import BiliAPI404Error, BiliAPIRetCodeError, BiliConnectionError, BiliDatebaseError, BiliInvalidRoomId, BiliInvalidShortUrl, BiliNoLiveRoom, BiliStatusCodeError
from nonebot.log import logger

__PLUGIN_NAME__ = "[bilibilibot~Client]"
class BiliClient():
    def __init__(self) -> None:
        self.__proxy_pool__ = [None]
        self.__retry_times__ = 3
        self.__ua_list__ = [
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML like Gecko) Chrome/44.0.2403.155 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.1 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2226.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2224.3 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.93 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062.124 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 4.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
            "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.125 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1944.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.3319.102 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2309.372 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2117.157 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1866.237 Safari/537.36",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.137 Safari/4E423F",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10",
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.517 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1667.0 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.16 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1623.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36",
            "Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1467.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1464.0 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1500.55 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36",
            "Mozilla/5.0 (X11; NetBSD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
            "Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.60 Safari/537.17",
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17"
        ]
        self.__usable_ip_list__ = set()

        self.API = {
            # 用于获取临时cookie
            "get_bili_cookie": "https://www.bilibili.com",
            "get_user_info_by_uid": "https://api.bilibili.com/x/space/acc/info?mid={}&jsonp=jsonp",
            "get_latest_video_by_uid": "https://api.bilibili.com/x/space/arc/search?mid={}&ps=1&tid=0&pn=1&order=pubdate&jsonp=jsonp",
            "get_live_info_by_room_id": "https://api.live.bilibili.com/room/v1/Room/get_info?room_id={}",
            "get_liver_info_by_uid": "https://api.live.bilibili.com/live_user/v1/Master/info?uid={}",
            "get_telegram_info_by_media_id": "https://api.bilibili.com/pgc/review/user?media_id={}",
            "get_telegram_info_by_ep_id": "https://api.bilibili.com/pgc/view/web/season?ep_id={}",
            "get_telegram_info_by_season_id": "https://api.bilibili.com/pgc/view/web/season?season_id={}",
            "get_telegram_latest_episode": "https://api.bilibili.com/pgc/view/web/season?season_id={}"
        
        }

        self.__proxy_lock__ = asyncio.Lock()

    @classmethod
    async def async_client_init(cls) -> "BiliClient":
        self = cls()
        await self.update_proxy_pool()
        return self


    def __request_header__(self) -> httpx.Headers:
        '''返回随机UA的header

        Returns:
            httpx.Headers: 返回header
        '''
        headers = {
            'User-Agent': choice(self.__ua_list__),
        }
        return httpx.Headers(headers)

    async def __get_page__(self, page: int):
        print(f'正在抓取第{page}页……')
        try:
            async with httpx.AsyncClient(headers=self.__request_header__()) as client:
                response = await client.get(url=f'http://www.ip3366.net/free/?stype=1&page={page}', timeout=5)
        except Exception as e:
            return
        text = response.text
        https_proxy_list = []
        html = etree.HTML(text)
        tr_list = html.xpath('/html/body/div[2]/div/div[2]/table/tbody/tr')
        for td in tr_list:
            ip_ = td.xpath('./td[1]/text()')[0] #ip
            port_ = td.xpath('./td[2]/text()')[0]  #端口
            type_ = td.xpath('./td[4]/text()')[0].lower()
            proxy = f"http://{ip_ + ':' + port_}"
            if type_ == "https":
                https_proxy_list.append(proxy)
        
        await asyncio.gather(*[self.test_ip(i) for i in https_proxy_list])

    async def __test_ip__(self, proxy) -> None:
        try:
            async with httpx.AsyncClient(headers=self.__request_header__(), proxies=proxy) as client:
                response = await client.get(url='https://api.live.bilibili.com/room/v1/Room/get_info?room_id=42062')
            if response.status_code == 200:
                if response.json()["code"] == 0:
                    self.__usable_ip_list__.add(proxy)
                    print(proxy, '\033[31m可用\033[0m')
            else:
                print(proxy, '不可用')
            return
        except Exception as e:
            print(proxy,'请求异常')
            return

    async def update_proxy_pool(self) -> None:
        '''更新代理池
        '''
        self.__usable_ip_list__.clear()
        tasks_list = [self.__get_page__(i) for i in range(1, 6)]
        await asyncio.gather(*tasks_list)

        async with self.__proxy_lock__:
            self.__proxy_pool__ = list(self.__usable_ip_list__)
            self.__proxy_pool__.append(None)
        
        self.__usable_ip_list__.clear()
    
    async def get_latest_video(self, uid: str, last_udpate_time: int) -> Tuple[bool, str, str, int, str]:
        """
        @description  :
        根据uid和时间戳, 返回元组，表示存在新视频或无新视频
        ---------
        @param  :
        uid: 查询新视频用户的uid
        last_udpate_time: 数据库中记录的最新视频的时间戳
        -------
        @Returns  :
        返回一个元组[是否更新，bv号，标题，发布时间戳，封面的链接]
        -------
        """
        try:
            async with httpx.AsyncClient(headers=self.__request_header__()) as client:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=self.API["get_latest_video_by_uid"].format(uid))
        except Exception as e:
            raise BiliConnectionError(0, uid, e.args[0])

        if response.status_code != 200:
            raise BiliStatusCodeError(0, uid, response.status_code)

        response = response.json()
        if response["code"] != 0:
            raise BiliAPIRetCodeError(0, uid, response["code"], response["message"])
        
        #logger.debug(f'{__PLUGIN_NAME__}{json.dumps(response, ensure_ascii=False, indent=4)}')
        
        latest_video = response['data']['list']['vlist'][0] if len(response['data']['list']['vlist']) != 0 else {}
        post_time = latest_video.get("created", 0)

        if post_time > last_udpate_time:
            # 发现新视频
            title = latest_video['title']
            bvID = latest_video['bvid']
            cover_url = latest_video['pic']
            return (True, bvID, title, post_time, cover_url)

        return (False, '', '', 0, '')

    async def init_up_info(self, uid: str) -> Tuple[str, int]:
        """
        @description  :
        根据uid查询up主信息
        ---------
        @param  :
        uid: 用户的uid
        -------
        @Returns  :
        返回一个元组
        [up主名字，最新视频的时间戳]
        -------
        """
        
        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(self.API["get_bili_cookie"])
                response1 = await client.get(url=self.API["get_user_info_by_uid"].format(uid))
            except Exception as e:
                raise BiliConnectionError(0, uid, e.args[0])

            if response1.status_code != 200:
                raise  BiliStatusCodeError(0, uid, response1.status_code)
    
            response1 = response1.json()
            if response1["code"] == -404:
                raise BiliAPI404Error()
            elif response1["code"] != 0:
                raise BiliAPIRetCodeError(0, uid, response1["code"], response1["message"])
            
            user_name = response1["data"]["name"]
            try:
                response2 = await client.get(url=self.API["get_latest_video_by_uid"].format(uid))
            except Exception as e:
                raise BiliConnectionError(0, uid, e.args[0])

            if response2.status_code != 200:
                raise  BiliStatusCodeError(0, uid, response2.status_code)
    
            response2 = response2.json()
            if response2["code"] != 0:
                raise BiliAPIRetCodeError(0, uid, response2["code"], response2["message"])
            
            latest_video = response2['data']['list']['vlist'][0] if len(response2['data']['list']['vlist']) != 0 else {}
            post_time = latest_video.get("created", 0)

            return (user_name, post_time)
           
    async def get_live_status(self, uid: str, room_id: str) -> Tuple[bool, str, str]:
        """
        @description  :
        根据房间号,获取直播间是否开播
        ---------
        @param  :
        uid: 主播的uid
        room_id: 直播间号
        -------
        @Returns  :
        返回一个元组
        [是否正在直播，直播间标题，直播间封面链接]
        -------
        """
        
        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(self.API["get_bili_cookie"])
                response = await client.get(self.API["get_live_info_by_room_id"].format(room_id))
            except Exception as e:
                raise BiliConnectionError(1, uid, e.args[0])
            
            if response.status_code != 200:
                raise BiliStatusCodeError(1, uid, response.status_code)
            
            response = response.json()
            if response["code"] != 0:
                raise BiliAPIRetCodeError(1, uid, response["code"], response["message"])
            
            live_status = response["data"]["live_status"]
            title = response["data"]["title"]
            cover_url = response["data"]["user_cover"]
        
            if live_status == 1:
                return (True, title, cover_url)
            else:
                return (False, "", "")
    
    async def init_liver_info(self, uid: str) -> Tuple[str, str]:
        """
        @description  :
        根据uid，返回直播间信息
        ---------
        @param  :
        uid: 用户的uid
        -------
        @Returns  :
        返回一个元组
        [用户名，直播间房间号]
        -------
        """

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(self.API["get_bili_cookie"])
                response = await client.get(self.API["get_liver_info_by_uid"].format(uid))
            except Exception as e:
                raise BiliConnectionError(0, uid, e.args[0])
            
        if response.status_code != 200:
            raise BiliStatusCodeError(1, uid, response.status_code)
        
        response = response.json()
        if response["code"] != 0:
            raise BiliAPIRetCodeError(1, uid, response["code"], response["message"])
        
        liver_name = response["data"]["info"]["uname"]
        room_id = response["data"]["room_id"]

        if room_id == 0:
            raise BiliNoLiveRoom(liver_name)
        
        return (liver_name, room_id)
    
    async def init_liver_info_by_room_id(self, room_id: str) -> Tuple[str, str]:
        '''根据直播房间号获得主播uid和主播用户名

        Args:
            room_id (str): 直播房间号

        Returns:
            Tuple[str, str]: 
            返回一个元组
            [主播uid, 主播用户名]
        '''

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response1 = await client.get(url=self.API["get_live_info_by_room_id"].format(room_id))
            except Exception as e:
                raise BiliConnectionError(1, f"房间号:{room_id}", e.args[0])
            
            if response1.status_code != 200:
                raise BiliStatusCodeError(1, f"房间号:{room_id}", response1.status_code)

            response1 = response1.json()
            if response1["code"] == 1:
                raise BiliInvalidRoomId(room_id)
            if response1["code"] != 0:
                raise BiliAPIRetCodeError(1, f"房间号:{room_id}", response1["code"], response1["message"])
            
            uid = str(response1["data"]["uid"])

            try:
                response2 = await client.get(self.API["get_liver_info_by_uid"].format(uid))
            except Exception as e:
                raise BiliConnectionError(0, uid, e.args[0])
            
            if response2.status_code != 200:
                raise  BiliStatusCodeError(1, uid, response2.status_code)
    
            response2 = response2.json()
            if response2["code"] != 0:
                raise BiliAPIRetCodeError(1, uid, response2["code"], response2["message"])
            
            liver_name = response2["data"]["info"]["uname"]

            return (uid, liver_name)
    
    async def init_telegram_info_by_ep_id(self, ep_id: int) -> Tuple[int, str, int, bool]:
        ''' 根据ep_id初始化番剧信息

        Args:
            ep_id (int): ep_id

        Returns:
            Tuple[int, str, int, bool]: 
            返回一个元组
            [season_id, 番剧名, 最新集, 是否完结]
        '''

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=self.API["get_telegram_info_by_ep_id"].format(ep_id))
            except Exception as e:
                raise BiliConnectionError(2, f"ep_id:{ep_id}", e.args[0])
            
            if response.status_code != 200:
                raise BiliStatusCodeError(2, f"ep_id:{ep_id}", response.status_code)
    
            response = response.json()
            if response["code"] == -404:
                raise BiliAPI404Error()
            elif response["code"] != 0:
                raise BiliAPIRetCodeError(2, f"ep_id:{ep_id}", response["code"], response["message"])

            season_id = response["result"]["season_id"]
            season_title = response["result"]["season_title"]
            is_finish = bool(response["result"]["publish"]["is_finish"])
            latest_episode = len(response["result"]["episodes"])
    


            return (season_id, season_title, latest_episode, is_finish)

    async def init_telegram_info_by_season_id(self, season_id: int) -> Tuple[int, str, int, bool]:
        ''' 根据season_id初始化番剧信息

        Args:
            season_id (int): season_id

        Returns:
            Tuple[int, str, int, bool]: 
            返回一个元组
            [season_id, 番剧名, 最新集, 是否完结]
        '''

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=self.API["get_telegram_info_by_season_id"].format(season_id))
            except Exception as e:
                raise BiliConnectionError(2, f"season_id:{season_id}", e.args[0])
            
            if response.status_code != 200:
                raise BiliStatusCodeError(2, f"season_id:{season_id}", response.status_code)
    
            response = response.json()
            if response["code"] == -404:
                raise BiliAPI404Error()
            elif response["code"] != 0:
                raise BiliAPIRetCodeError(2, f"season_id:{season_id}", response["code"], response["message"])

            season_title = response["result"]["title"]
            is_finish = bool(response["result"]["publish"]["is_finish"])
            latest_episode = len(response["result"]["episodes"])

            return (season_id, season_title, latest_episode, is_finish)

    async def init_telegram_info_by_media_id(self, media_id: int) -> Tuple[int, str, int, bool]:
        ''' 根据media_id初始化番剧信息

        Args:
            media_id (int): media_id

        Returns:
            Tuple[int, str, int, bool]: 
            返回一个元组
            [season_id, 番剧名, 最新集, 是否完结]
        '''

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=self.API["get_telegram_info_by_media_id"].format(media_id))
            except Exception as e:
                raise BiliConnectionError(2, f"media_id:{media_id}", e.args[0])
            
            if response.status_code != 200:
                raise BiliStatusCodeError(2, f"media_id:{media_id}", response.status_code)
    
            response = response.json()
            if response["code"] == -404:
                raise BiliAPI404Error()
            elif response["code"] != 0:
                raise BiliAPIRetCodeError(2, f"media_id:{media_id}", response["code"], response["message"])

            season_id = response["result"]["media"]["season_id"]
            season_title = response["result"]["media"]["title"]
            is_finish = False
            latest_episode = int(response["result"]["media"]["new_ep"]["index"])

            return (season_id, season_title, latest_episode, is_finish)
    
    async def get_telegram_latest_episode(self, season_id: int, index: int) -> Tuple[bool, int, str, str, str, bool]:
        '''根据season_id获取番剧的最新集信息

        Args:
            season_id (int): season_id
            index (int): 记录的最新集数

        Returns:
            Tuple[bool, int, str, str, str, bool]: 
            返回一个元组
            [是否更新, 最新集数, 最新集标题, 最新集链接, 封面链接, 是否完结]
        '''

        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=self.API["get_telegram_latest_episode"].format(season_id))
            except Exception as e:
                raise BiliConnectionError(2, f"season_id:{season_id}", e.args[0])
            
            if response.status_code != 200:
                raise BiliStatusCodeError(2, f"season_id:{season_id}", response.status_code)
    
            response = response.json()
            if response["code"] != 0:
                raise BiliAPIRetCodeError(2, f"season_id:{season_id}", response["code"], response["message"])

            episodes = response['result']['episodes']
            is_finish = bool(response["result"]["publish"]["is_finish"])

            if len(episodes) > index:
                latest_episode = episodes[-1]
                cover_url = latest_episode['cover']
                title = latest_episode['long_title']
                play_url = latest_episode['share_url']
                
                return (True, len(episodes), title, play_url, cover_url, is_finish)
            else:
                return (False, 0, "", "", "", is_finish)
    
    async def parse_short_url(self, short_url: str) -> Tuple[int, str]:
        '''解析b23.tv的短链接,返回短链接类型以及目标id

        Args:
            short_url (str): 短链接

        Returns:
            Tuple[int, str]: 
            返回一个元组
            [类型:0-up的uid,1-主播的房间号,2-番剧的ep_id, 目标id]
        '''
        async with httpx.AsyncClient(headers=self.__request_header__()) as client:
            try:
                await client.get(url=self.API["get_bili_cookie"])
                response = await client.get(url=short_url)
            except Exception as e:
                raise BiliConnectionError(3, short_url, e.args[0])
            
            if response.status_code != 302:
                raise BiliStatusCodeError(3, short_url, response.status_code)
            
            origin_url = response.headers["Location"].split("?")[0]
            logger.debug(f"get origin url = {origin_url}")
            target_id = origin_url.split("/")[-1]
            if "space" in origin_url:
                return (0, target_id)
            elif "live" in origin_url:
                return (1, target_id)
            elif "bangumi" in origin_url:
                return (2, target_id)
            
            raise BiliInvalidShortUrl(short_url)
    
bili_client = BiliClient()